package media.config;

import java.io.File;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.StreamTokenizer;
import java.io.StreamCorruptedException;
import java.io.FileNotFoundException;
import javax.media.format.AudioFormat;
import javax.media.format.VideoFormat;
import media.gui.*;
import media.network.*;
import media.common.User;

/**
 *
 * Title:        Config.java<br>
 * Description:  Keeps track of configuration file<br>
 * Copyright:    Konrad Rzeszutek<br>
 * Company:      darnok.dhs.org<br>
 * Version:      $Id: Config.java,v 1.4 2000/11/28 03:41:24 konrad Exp $<br><br>
 *
 * Keeps the configuration of the VideoConference package.
 */
public class Config {

    /* Defaults.  */

  private String CONFIG_FILE   = "config.ini";
  private String DEFAULT_DRV   = "file";

  public String DEFAULT_AUDIO_ENC = AudioFormat.GSM_RTP;
  public String DEFAULT_VIDEO_ENC = VideoFormat.JPEG_RTP;

  /* Schemes */
  public String SCHEME_DIR    = "schemes";
  public String SCHEME        = "Bert is EVIL";
  public Schemes SCHEMES_OBJ  = null;
  public Scheme SCHEME_OBJ    = null;

  /* Ports */
  public int  VIDEO_PORT      = 40910;
  public int  AUDIO_PORT      = 40912;

  /* Video Drivers */
   public String VIDEO_DRIVER  = "file";

  /* Audio Driver */
  public String AUDIO_DRIVER  = "file";

  /* Inputs/Outputs */
  public String VIDEO_INPUT   = "files/movie.mov";
  public String AUDIO_INPUT   = "files/sound.gsm";

  public String[] VIDEO       = null;
  public String[] AUDIO       = null;

  /* These are used when the user wants to connect */
  // This needs to be changed whenever we have a connection
  public String RECV_HOSTNAME = "224.144.251.104";
  public int RECV_AUD_PORT    = 40910;
  public int RECV_VID_PORT    = 40912;
  public int TTL              = 1;

  public String SEND_HOSTNAME = "224.144.251.104";
  public int SEND_AUD_PORT    = 40810;
  public int SEND_VID_PORT    = 40812;

  public final int I_AM_BUSY      = 32;
  public final int I_AM_AVAILABLE = 829;

  public int STATE	      = I_AM_AVAILABLE;

  public OcularBase progress  = null;
  public DeviceFinder df     = null;
    public User oldyou         = null;

    // Ping-Pong interval
    public long INTERVAL       = 2000L;
  /*
   * Reads the configuration file and sets the defaults.
   * If it can't find the configuration file, it creates one (with default settings)
   * and re-reads the config file.<br>
   * <b>Parameters</b>:<br><br>
   *  OcularBase (<i>p</i>) is used only for its <i>write</i> method.<br>
   * <i>p</i> can be <u>null</u>
  */
  public Config(OcularBase p) throws StreamCorruptedException,
                                     FileNotFoundException {

    this.progress = p;
    configFile = new File(CONFIG_FILE);

    if (!configFile.exists())
      writeConfigFile();

    readConfigFile();
    df = new DeviceFinder();
    detectDevices();
  }

  /**
   * Writes out status messages. If the <i>Ocularbase</i> has registered
   * in the constructor, the contents will be pased to <i>OcularBase</i>,
   * otherwise the message will be printed out to the system's default
   * console.
   * <br><b>Parameters</b>:<br>
   *  s != null
   */
  public void write (String s) {

    if (progress == null)
      System.out.println(" ["+s+" ...]");
    else
      progress.write(s);
  }

    public void error (String s) {

	System.err.println("* "+s+" ...");
    }
  /**
   * Creates a new configuration file with the object current settings.
   * This method writes the <u>config.ini</u> file into the directory
   * from where this program has been run. The contents of the configuration
   * file stricly depends on the state of this class.
   * <br><br><i>StreamCorruptedException</i> is thrown when the file can't
   * be written to the medium.
   */
   public void writeConfigFile() throws StreamCorruptedException{

    BufferedWriter writer = null;
    try {
      writer = new BufferedWriter(new FileWriter(configFile));

      writer.write("Scheme       \""+SCHEME+"\"");
      writer.newLine();
      writer.write("SchemeDir    \""+SCHEME_DIR+"\"");
      writer.newLine();
      writer.write("VideoPort    "+VIDEO_PORT);
      writer.newLine();
      writer.write("AudioPort    "+AUDIO_PORT);
      writer.newLine();
      writer.write("VideoInput   \""+VIDEO_INPUT+"\"");
      writer.newLine();
      writer.write("AudioInput   \""+AUDIO_INPUT+"\"");
      writer.newLine();
      writer.write("VideoDriver  \""+VIDEO_DRIVER+"\"");
      writer.newLine();
      writer.write("AudioDriver  \""+AUDIO_DRIVER+"\"");
      writer.close();

    } catch (Exception e) {
      // This sucks.
	System.out.println(e);
      throw new StreamCorruptedException("Can't write to a 'config.ini' file!");
    }
    write("Wrote "+CONFIG_FILE);
   }

   /**
    * Reads the configuration file and sets the state of this object to whatever
    * the contents of the config file was. <br><br>
    * <i>StreamCorruptedException</i> is thrown when the syntax of the
    * file  is not correct.
    */
    public void readConfigFile() throws  StreamCorruptedException   {

      BufferedReader reader = null;
      StreamTokenizer st = null;
      try {
        reader = new BufferedReader(new FileReader(configFile));
        st = new StreamTokenizer(reader);
        String manipulate = "";
        String match = "";
        while (st.nextToken() != StreamTokenizer.TT_EOF) {
          manipulate = st.sval.toLowerCase();
          for (int loop0 = 0; loop0 < match_str.length; loop0++)
          if (match_str[loop0].compareTo(manipulate)==0) {// match found
              st.nextToken();

              switch (loop0) {
                // 'scheme'
                case 0: { SCHEME = st.sval; break; }
                case 1: { SCHEME_DIR = st.sval; break; }
                // 'videoport' and 'audioport'
                case 2: { VIDEO_PORT = (int)st.nval; break; }
                case 3: { AUDIO_PORT = (int)st.nval; break; }
                // 'videoinput' and 'audioinput'
                case 4: { VIDEO_INPUT = st.sval; break; }
                case 5: { AUDIO_INPUT = st.sval; break; }
                case 6: { VIDEO_DRIVER = st.sval;break; }
                case 7: { AUDIO_DRIVER = st.sval; break;}
              }
              loop0 = match_str.length+1;
          }
        }
        reader.close();
        write("Read "+CONFIG_FILE);
      } catch (Exception e) {
        // Error reading the file - ?
         throw new StreamCorruptedException("Wrong format in "+CONFIG_FILE+"!");
      }

      // CHECK! FRAGILE!
      try {
        write("Reading schemes");
        SCHEMES_OBJ  = new Schemes(SCHEME_DIR, this);
        write("Reading default scheme");
        SCHEME_OBJ   = SCHEMES_OBJ.getScheme(SCHEME);
        if (SCHEME_OBJ == null) {
          throw new FileNotFoundException("Couldn't find the default scheme: ["+SCHEME+"]!");
        }
      } catch (Exception e) {

        throw new StreamCorruptedException(e.getMessage());
      }
    }

    /**
     * Detects all audio/video devices.
     * Checks for video/audio drivers and based on their availability adds
     * (or not) to VIDEO/AUDIO array.
     */
     public void detectDevices () {

        write("Detecting devices");
        df.requeryDevices();         //Requeries for devices

        AUDIO = df.getAudioDeviceNames();  //Detects Audio Devices
        VIDEO = df.getVideoDeviceNames();

        write("Finished detecting devices");
     }

  private String[] match_str  = {"scheme",
                                 "schemedir",
                                 "videoport",
                                 "audioport",
                                 "videoinput",
                                 "audioinput",
                                 "videodriver",
                                 "audiodriver"};
  private File configFile;
}















