/**
 * Title:        AudioSender.java<p>
 * Description:  Allows you to send audio using rtp.  Some code was taken from VideoTransmit.java on
 *               the sun website and some was taken from the java media services documentation<p>
 * Copyright:    Copyright (c) George Voulgarakis<p>
 * Company:      George's Company<p>
 * @author:      George Voulgarakis
 * @version $Id: AudioSender.java,v 2.0 2000/11/20 07:08:02 George Exp $
 */

package media.network;

import media.config.*;
import media.gui.*;
import javax.media.*;
import java.io.*;
import javax.media.protocol.*;
import javax.media.control.*;
import javax.media.format.*;
import javax.media.rtp.*;
import javax.media.rtp.event.*;
import javax.media.rtp.rtcp.*;
import java.net.*;
import java.util.*;
import com.sun.media.rtp.*;

public class AudioSender implements ControllerListener {

   private Config config             = null; //an instance of the Config class
   private OcularBase gui            = null;
   private String source             = null; //the media location
   private Processor processor       = null; //the processor
   private DataSink dataSink         = null; //creates an rtp datasink
   private DataSource dataSource     = null; //the datasource
   private MediaLocator ml           = null; //the media locator

   //Constants to set up output format of audio

   private static final int sampleRate = 44100;  //sample rate of sound
   private static final int sampleSize = 4;  //sample size in bits
   private static final int channels = 1;  //number of channels

   public static final int OK             = 2324897;
   public static final int ERROR          = 23489732;
    private boolean isFile = false;
  /**
   *  Creates a new Audio sender with the config, ocularbase, destination ipaddress, and destination port specified
   */

   public AudioSender(OcularBase gui,Config config ) {

      this.gui = gui;
      this.config = config;

   }


   /**
    * Creates a processor and sets it up
    */

    private void configureProcessor(MediaLocator ml) throws NoProcessorException,IOException, NotRealizedError, MediaException {


       processor = javax.media.Manager.createProcessor(ml);

       processor.configure(); //change processor mode to configured

       //Infinite loop while the processor is still configuring
       while(processor.getState() ==Processor.Configuring);

       TrackControl tracks[] = processor.getTrackControls(); //gets the track controller of the processor

       boolean encodingOk = false;  //the encoding is not okay

       for (int i = 0; i < tracks.length; i++){
         if(!encodingOk && (tracks[i] instanceof FormatControl)){
             Format f = ((FormatControl)tracks[i]).getFormat();

             if (f instanceof AudioFormat) {
               tracks[i].setFormat(new AudioFormat(config.DEFAULT_AUDIO_ENC));
               tracks[i].setEnabled(true);
	       encodingOk = true;
             }
         } else
             tracks[i].setEnabled(false); //Disable it because it is not the right format
       }

       if(encodingOk){
          processor.realize();   //the processors mode is changed to realized
          //Infinite loop while waiting for the processor to be realized

          while (processor.getState() != Processor.Realized) ;
	  dataSource = processor.getDataOutput(); //gets the data output

       }
       processor.addControllerListener(this);
       String rtpURL = "rtp://" + config.SEND_HOSTNAME + ":" + config.SEND_AUD_PORT +"/audio";
       MediaLocator outputLocator = new MediaLocator(rtpURL);

       dataSink = javax.media.Manager.createDataSink(dataSource,outputLocator);
       dataSink.open();     //opens the datasink
       dataSink.start();  //starts the datasink

       dataSource.start(); //starts the datasource
       processor.start();  //starts the processor
   }

    /**
     * Starts Transmission of data
     */

    public int startAudioSender(){

      if (this.config.AUDIO_DRIVER.equals("file"))
          this.source = "file:///" + config.AUDIO_INPUT;
      else
          this.source = config.AUDIO_DRIVER;

      
      try {
	  if (this.source.substring(0,8).equals("file:///")) {
            URL url = new URL(this.source);
            ml = new MediaLocator(url);
	    isFile  = true;
	  }
	  else {
	      ml = new MediaLocator(config.df.getAudioLocator(this.source));
	     }
      } catch(MalformedURLException e) {
              if (gui.error("Invalid audio input - doesn't exit"))
                return ERROR;
      }

      config.write("Getting Audio from: "+ml);
      try {
        this.configureProcessor(ml); //configures processor and starts a datasink if successful
      } catch (Exception e) {
          if (gui.error(e.getMessage()))
            return ERROR;
      }
      return OK;
    }

   /**
    * Closes Connection and Datasink
    */

   public int resetAudio(){

      try {
        processor.stop();
        processor.close();
        dataSink.stop();
        dataSink.close();
        processor = null;
        dataSink = null;
	isFile = false;
      } catch (Exception e) {

        return ERROR;
      }

      return OK;
   }

    public synchronized void controllerUpdate (ControllerEvent event){
	

	if(event instanceof EndOfMediaEvent){
	    
	    if(isFile == true){
		processor.setMediaTime(new Time(0));
		processor.start();
          }      
      }
    }
}



