/**
 * Title:        VideoSender.java<p>
 * Description:  Allows you to send video using rtp.  Some code was taken from VideoTransmit.java on
 *               the sun website and some was taken from the java media services documentation<p>
 * Copyright:    Copyright (c) George Voulgarakis<p>
 * Company:      George's Company<p>
 * @author:      George Voulgarakis
 * @version $Id: VideoSender.java,v 2.0 2000/11/20 07:08:02 George Exp $
 */

package media.network;

import media.config.*;
import media.gui.*;
import javax.media.*;
import java.io.*;
import javax.media.protocol.*;
import javax.media.control.*;
import javax.media.format.*;
import javax.media.rtp.*;
import javax.media.rtp.event.*;
import javax.media.rtp.rtcp.*;
import java.net.*;
import java.util.*;
import com.sun.media.rtp.*;
import java.awt.Dimension;

public class VideoSender implements ControllerListener {

   private Config config         = null; //an instance of the Config class
   private OcularBase ocularBase = null;  //an instance of OcularBase class


   private String source         = null;   //the media location
   public final static int OK    = 12897897;
   public final static int ERROR = 1892;

   private Processor processor   = null;  //the processor
   private DataSink dataSink     = null;  //creates an rtp datasink
   private DataSource dataSource = null;  //datasource
   private MediaLocator ml       = null;  //the media locator
   private OcularBase gui        = null;
   public Player player        = null;
    private boolean isFile = false; //if it is a file 
    public int width             = 320;
    public int height             = 240;
    public float frameRate        = 2;
  /**
   *  Creates a new Video sender with the config, ocularbase, destination ipaddress, and destination port specified
   */

   public VideoSender( OcularBase gui, Config config) {

      this.config = config;
      this.gui = gui;


   }


   /**
    * Creates a processor and sets it up
    */

    private void configureProcessor(MediaLocator ml) throws NoProcessorException, IOException, NotRealizedError, MediaException {



        processor = javax.media.Manager.createProcessor(ml);

        processor.configure(); //change processor mode to configured

       //Infinite loop while the processor is still configuring
        while(processor.getState() ==Processor.Configuring) ;

        TrackControl tracks[] = processor.getTrackControls(); //gets the track controller of the processor

        boolean encodingOk = false;  //the encoding is not okay

       for (int i = 0; i < tracks.length; i++){
	   if(!encodingOk && (tracks[i] instanceof FormatControl)){
             Format f = ((FormatControl)tracks[i]).getFormat();
             if (f instanceof VideoFormat) {
		 //	       boolean good = checkVideo(tracks[i].getFormat());
		 //System.out.println("Data is "+good);
		 //		 tracks[i].setFormat(new VideoFormat(config.DEFAULT_VIDEO_ENC,new Dimension(this.width,this.height),Format.NOT_SPECIFIED,Format.byteArray,frameRate));
               
		  tracks[i].setFormat(new VideoFormat(config.DEFAULT_VIDEO_ENC));
               tracks[i].setEnabled(true);
                encodingOk = true;
	     }
         } else
	 tracks[i].setEnabled(false); //Disable it because it is not the right format 
       }

       if(encodingOk){

          /* BOOM, this is where it crashes */
          processor.realize();   //the processors mode is changed to realized
          //Infinite loop while waiting for the processor to be realized
          while (processor.getState() != Processor.Realized) ;

	  setJPEGQuality(processor, 0.5f);
          dataSource = processor.getDataOutput(); //gets the data output

       }
       processor.addControllerListener(this);

          String rtpURL = "rtp://" + config.SEND_HOSTNAME + ":" + config.SEND_VID_PORT +"/video";

          MediaLocator outputLocator = new MediaLocator(rtpURL);

           dataSink = javax.media.Manager.createDataSink(dataSource,outputLocator);
	   // player = javax.media.Manager.createPlayer(dataSource);
	   //player.addControllerListener(this);
	   //player.realize();
           dataSink.open();     //opens the datasink

           dataSink.start();  //starts the datasink

           dataSource.start(); //starts the datasource

           processor.start();  //starts the processor
	
   }

 /**


     * Setting the encoding quality to the specified value on the JPEG encoder.


     * 0.5 is a good default.


     */


    void setJPEGQuality(Player p, float val) {





	Control cs[] = p.getControls();


	QualityControl qc = null;


	VideoFormat jpegFmt = new VideoFormat(VideoFormat.JPEG);






	// Loop through the controls to find the Quality control for


 	// the JPEG encoder.


	for (int i = 0; i < cs.length; i++) {





	    if (cs[i] instanceof QualityControl &&


		cs[i] instanceof Owned) {


		Object owner = ((Owned)cs[i]).getOwner();





		// Check to see if the owner is a Codec.


		// Then check for the output format.


		if (owner instanceof Codec) {


		    Format fmts[] = ((Codec)owner).getSupportedOutputFormats(null);


		    for (int j = 0; j < fmts.length; j++) {


			if (fmts[j].matches(jpegFmt)) {


			    qc = (QualityControl)cs[i];


	    		    qc.setQuality(val);



			    break;


			}


		    }


		}


		if (qc != null)


		    break;


	    }


	}


    }



   boolean checkVideo(Format original) {





	int width, height;


	boolean isRight = false;
	Dimension size = ((VideoFormat)original).getSize();
	Format jpegFmt = new Format(VideoFormat.JPEG_RTP);
	VideoFormat h263Fmt = new VideoFormat(VideoFormat.H263);
	if (original.matches(jpegFmt)) {


	    // For JPEG, make sure width and height are divisible by 8.


	    if ( (size.width % 8 == 0) && (size.height % 8 == 0))
		isRight = true;
	    else 
		isRight = false;
	} else if (original.matches(h263Fmt)) {



	    isRight = false;
	} else {


	    // We don't know this particular format.  We'll just


	    // leave it alone then.


	    isRight = false;
	}
	return isRight;
   }
    /**
     * Starts Transmission of data
     */

    public int startVideoSender(){

     if (this.config.VIDEO_DRIVER.equals("file"))
          this.source = "file:///" + config.VIDEO_INPUT;
      else
          this.source = config.VIDEO_DRIVER;
     config.write("Getting video from: "+source);

      try {

	  if(this.source.substring(0,8).equals("file:///")) {
	    this.isFile = true;
            URL url = new URL(this.source);
            ml = new MediaLocator(url);
          }
          else {

	      ml = new MediaLocator(config.df.getVideoLocator(this.source));
          }
      } catch(MalformedURLException e) {
              if (gui.error("Invalid video input - doesn't exist!"))
                return ERROR;
      }


      try {
        this.configureProcessor(ml); //configures processor and starts a datasink if successful
      } catch (Exception e) {
          if (gui.error(e.getMessage()))
            return ERROR;
      }
      return OK;

    }

   /**
    * Closes Connection and Datasink
    */

   public int resetVideo(){

    try {

	processor.stop();
        processor.close();
        dataSink.stop();
        dataSink.close();
        isFile = false;
        processor = null;
        dataSink = null;

    } catch (Exception e) {

        return ERROR;
      }

      return OK;
   }
   
   
    /**
    *  Implements Controller Listeners controller update method
    */

    public synchronized void controllerUpdate (ControllerEvent event){


      if(event instanceof EndOfMediaEvent){

	  if(isFile == true){
	      processor.setMediaTime(new Time(0));
              //System.out.println("Media Time" + processor.getMediaTime());
              //processor.prefetch();

              processor.start();

          }      
      }
    }

}




